import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import javax.naming.*;
import javax.swing.tree.*;
import java.util.*;
import org.omg.CORBA.SystemException;


/** Display Objects on a JNDI in a handy tree
@author Jonathan Hallam
*/

/* Modified by Richard Johnson 19/09/00 to add modifications that were made to */
/* Andy Clement's listJNDI */


class JNDIview extends JFrame{
	/**Constructs the main frame and displays it. Will base the tree on the first in the list, which may cause problems if cannot connect to it.
	@param title The title to give the window.
	*/
	public JNDIview(String title, String initialURL) throws NamingException{
		super(title);
		
		ErrorHandler.init(this); //initialise the error hadler (it just a reference to this frame
		contentPane = getContentPane();
		
		int index=0; //the default index in the URLproviders list to use first
		//see if initialURL is an index into URLproviders
		try{
			int t;
			t=Integer.parseInt(initialURL);	
			if(t>=0 && t<providerURLs.length) index=t;
			initialURL="";
		}catch(NumberFormatException e) {}


		if(!initialURL.equals("")){ //have a URL for a server - see what to do with it
			//first see if initialURL is in the list
			for(int i=1; i!=providerURLs.length; i++) 
				if(initialURL.equals(providerURLs[i])) index=i;

		  	if(index==0 && !initialURL.equals(providerURLs[0]) ) { //not found, so need to add it to the beginning of the list
				String t[]=providerURLs;
				providerURLs=new String[t.length+1];
				System.arraycopy(t,0, providerURLs,1, t.length);
				providerURLs[0]=initialURL;
		 	}
	 	}

		//set up sys. properties and get initial context
		Context ctx = getInitialContext(providerURLs[index]);

		DefaultMutableTreeNode root=new JNDINode(ctx,ctx.toString()); //create the root node

		JTree tree = new JTree(root); //generate  tree
		
		//add listener to expand the tree
		tree.addTreeExpansionListener( new ExpansionListener(tree) );

		treeScroll=new JScrollPane(tree); //generate components
		provideURL=new JComboBox(providerURLs);
		provideURL.setEditable(true);
		provideURL.setSelectedIndex(index);
		provideURL.addActionListener( new ComboListener((JNDINode)root, this, tree) );

		contentPane.add(treeScroll, BorderLayout.CENTER);
		contentPane.add(provideURL, BorderLayout.NORTH);
		setBounds(100,100, 500, 600);
		setVisible(true);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	}
	
	/**Method to get an initial context from a particular URL provider. Changes System properties. Will exit on any exception while getting the context.
	@param urlProvider The url provider to get the context from.
	*/
	public Context getInitialContext(String urlProvider){
		Properties p = System.getProperties();
		String contextFactory = (String) p.get(Context.INITIAL_CONTEXT_FACTORY);

		if(contextFactory == null || contextFactory.equals("")) 
		{
			contextFactory = "com.sun.jndi.cosnaming.CNCtxFactory";
			//contextFactory = "com.ibm.ejs.ns.jndi.CNInitialContextFactory";
		}
	
	
		p.put(Context.INITIAL_CONTEXT_FACTORY,contextFactory); //set the context factory
		p.put(Context.PROVIDER_URL,urlProvider); //set the url providers
	
		System.setProperties(p); //update properties
		Context ctx=null; 
		try{
			ctx = new InitialContext(); //grab the context
		}catch(Exception ex) {
			ErrorHandler.error(ex.getMessage()+"\nProgram will now close");
			System.exit(1); //if do not have a context, cannot get very far.
		}

		return ctx;
	}		


	/**Simple main that will set up the window, and install a close handler.
	*/
	public static void main(String argv[]) throws NamingException{
		String URL="";
		if(argv.length==1) URL=argv[0];
		JFrame f=new JNDIview("JNDI viewer 1.2",URL); //create frame
		f.addWindowListener( //do shut down properly
			new WindowAdapter() {//ANONYMOUS CLASS
				public void windowClosed(WindowEvent e) {
					System.exit(0);
			 	}
			}
		);// END OF ANONYMOUS CLASS **************
	}

	
	String providerURLs[]={"iiop://cicsfvejb1.hursley.ibm.com:900/","iiop://sweep.hursley.ibm.com:900/",
  								"iiop://cocopah.hursley.ibm.com:900/","iiop://pushmepullyou.hursley.ibm.com:900/"};
	JComboBox provideURL;
	JScrollPane treeScroll;
	Container contentPane;
}



class JNDINode extends DefaultMutableTreeNode{
	public boolean isExplored=false;
	private String name;
	
	
	public JNDINode(Object o, String n){
		//this messes the names around. Nicked from the listJNDI code, and I haven't looked at it closely to see what it does.
		setUserObject(o);


            String s2=n;
            while (s2.indexOf("\\.")!=-1)
            {
                s2=new String(s2.substring(0,s2.indexOf("\\."))+s2.substring(s2.indexOf("\\.")+1));
            }
            if (s2.indexOf(".")==-1)
            {
                name=s2;
            }
            else
            {
                String s3=s2.substring(0,s2.indexOf("."));
                String s4=s2.substring(s2.indexOf(".")+1);
                name=s3;
                int i=s3.length();
                while ((i++)<34) name+=" ";
                name+=s4;
            }
        }

	public boolean getAllowsChildren() { return isContext(); }
	public boolean isLeaf() {return !isContext(); }
	public boolean isExplored() {return isExplored;}

	private boolean isContext(){
		return (getUserObject() instanceof Context);
  	}

	public String toString() {
		return name;
	}

	public void clean(){
		if( !isContext() ) return;
		//System.out.println("Cleaning");
		if(isExplored) removeAllChildren(); //get rid of all of them
		isExplored=false;
 	}

	/**This method adds the children that should exist for this node if it is a Context.
	*/
	public void explore(){
		try{
			if(!isContext()) 
				return; //if it is not a context, no exploring to be done
		 	
		 	if(!isExplored) { //if haven't already explored, explore
				Context cxt=(Context)getUserObject(); //get the context
				NamingEnumeration list=cxt.listBindings(""); //get list of Bindings
				Binding b;
				int c=0;
				while(list.hasMore()){ //loop through bindings and add a node for each one
					try{
				 		//*************************** Next Line Locks Up Sometimes ************
						b=(Binding)list.next(); //locks up here, trying to find out how to set timeout (if possible)
						add(new JNDINode(b.getObject(),b.getName()));
			 		}catch(SystemException e){ //this will catch all CORBA errors
						add(new JNDINode(e, e.toString()) ) ; //add it to the tree, but finish
 	  				}
			 	}

	  		}
	 	}catch(SystemException e){ //this will catch all CORBA errors
			add(new JNDINode(e, e.toString()) ) ; //add it to the tree, but finish
 	  	}catch(Exception ex){
			ErrorHandler.error(ex.toString()); //Any exceptions here should only stop children being added, so should be safe
	  		Class cls=ex.getClass();
			System.out.println(cls.getName());
		}
		isExplored = true; //do not want to do it again
	}
}



class ComboListener implements ActionListener{
	public ComboListener(JNDINode rt, JNDIview b, JTree t){
		root=rt;
		base=b;
		tree=t;
  	}
	public void actionPerformed(ActionEvent e){
		//System.out.println("Helle");
		Context ctx=base.getInitialContext((String)(((JComboBox)e.getSource()).getSelectedItem()));
		root=new JNDINode(ctx, ctx.toString() );
		Container p=tree.getParent(); //get the container holding the tree
		p.remove(tree); //remove the tree
		tree=new JTree(root); //create a new tree
		tree.addTreeExpansionListener( new ExpansionListener(tree) ); //add a new listener to this tree
		p.add(tree); //add the tree to the container
		p.repaint(); //repaint the container
	}
	private JNDIview base;	
 	private JNDINode root;
	private JTree tree;
}


class ExpansionListener implements TreeExpansionListener{
	public ExpansionListener(JTree t){
		tree=t;
  	}
	public void treeCollapsed(TreeExpansionEvent e) {
		//this performs a clean up
		TreePath path=e.getPath();
		JNDINode node=(JNDINode)path.getLastPathComponent();
		if(node.isExplored()){ //if explored, want to tidy
			DefaultTreeModel model= (DefaultTreeModel)tree.getModel(); //get the underlying model
			node.clean(); //add the children nodes
			model.nodeStructureChanged(node); //alter the tree model	
	 	}
	}
	public void treeExpanded(TreeExpansionEvent e){
		//System.out.print("Expanding");
		tree.setCursor(new Cursor(Cursor.WAIT_CURSOR)); //change to a wait cursor
		//System.out.println(
		TreePath path=e.getPath(); //get the path to the expanded node
		JNDINode node=(JNDINode)path.getLastPathComponent(); //get the actual node
		//System.out.println(" node "+node.toString());
		if(!node.isExplored()) { //if not explored
			DefaultTreeModel model= (DefaultTreeModel)tree.getModel(); //get the underlying model
			node.explore(); //add the children nodes
			model.nodeStructureChanged(node); //alter the tree model
	 	}
		//System.out.println("Finished Expanding");
		tree.setCursor(new Cursor(Cursor.DEFAULT_CURSOR)); //set the cursor back
	}
	JTree tree;
}


class ErrorHandler{
	private static JFrame frame=null;
	public static void init(JFrame f){
		frame=f;
  	}
	public static void error(Exception ex){
		error(ex.getMessage());
  	}

	public static void error(String message){
		JOptionPane.showMessageDialog(frame, "This error has occured: "+message,"JNDIview Error",JOptionPane.ERROR_MESSAGE);
	}
}
