package itso.cics.eci.j2ee.testercci;

import com.ibm.connector2.cics.*;
import com.ibm.ctg.client.T;

import javax.resource.cci.*;
import javax.resource.*;
import javax.naming.InitialContext;
import javax.ejb.*;
import java.io.PrintWriter;

/**
 * Bean implementation class for Enterprise Bean: CTGTesterCCI
 * @version 1.02
 */
public class CTGTesterCCIBean implements SessionBean
{
    private SessionContext mySessionCtx;
    transient private Connection eciConn;
    transient private Interaction eciInt;
    transient private ECIInteractionSpec eSpec;
    
    /**
     * getSessionContext
     */
    public SessionContext getSessionContext()
    {
        return mySessionCtx;
    }
    
    /**
     * setSessionContext
     */
    public void setSessionContext(SessionContext ctx)
    {
        mySessionCtx = ctx;
    }
    
    /**
     * ejbActivate
     */
    public void ejbActivate()
    {
    }
    
    /**
     * ejbCreate
     */
    public void ejbCreate() throws CreateException
    {
        // create an Interaction spec to work with, This is specific to ECI
        eSpec = new ECIInteractionSpec();
    }
    
    /**
     * ejbPassivate
     */
    public void ejbPassivate()
    {
    }
    
    /**
     * ejbRemove
     */
    public void ejbRemove()
    {
        eSpec = null;
    }
    
    /**
     * This is the main method that does the call to the CICS Server.
     * @param funcName the CICS program to execute
     * @param encoding the encoding to use on the commarea data
     * @param commarea the commarea data to send to the CICS program
     * @param commareaLength length of the commarea
     * @param username the CICS username to use
     * @param password the CICS user password to use
     * @param managed whether to use a managed resource adapter or create one
     * @param gatewayURL URL of the Gateway daemon to connect to, for unmanaged
     * @param gatewayPort port of the Gateway daemon to connect to, for unmanaged
     * @param cicsServer the CICS server connection to send the request to for unmanaged
     * @param mirror the CICS mirror transaction to use for unmanaged
     * @param trace the trace level to use for unmanaged
     * @param iterations the number of times to execute the transaction
     * @param appTrace whether to trace using T class
     * @returns ResultsBean containing the string results and the binary data, or null if something went wrong but no exception occured.
     */
    public ResultsBean execute(
        String funcName,
        String encoding,
        String commarea,
        int commareaLength,
        String username,
        String password,
        boolean managed,
        String gatewayURL,
        int gatewayPort,
        String cicsServer,
        String mirror,
        int trace,
        int iterations,
        boolean appTrace)
        throws ResourceException, Exception
    {
        JavaStringRecord jsr = null;
        if(managed)
        {
            getConnection();
        }
        else
        {
            getNonManagedConnection(
                username,
                password,
                gatewayURL,
                gatewayPort,
                cicsServer,
                mirror,
                trace);
        }
        
        // alter application trace
        T.setTimingOn(true);
        /*
          The managed resource adapter will not output any tracing unless
          T debug is enabled. Therefore enabling debug will not only debug
          Application side methods, but the CCI trace at the level set in
          the managed environment.
         */
        if (appTrace)
        {
            T.setDebugOn(true);
        }
        else
        {
            T.setDebugOn(false);
        }

        // setup the interactionSpec. 
        eSpec.setFunctionName(funcName);
        eSpec.setCommareaLength(commareaLength);
        // set reply length to same size as commarea
        eSpec.setReplyLength(commareaLength);
        eSpec.setInteractionVerb(ECIInteractionSpec.SYNC_SEND_RECEIVE);

        if (iterations < 1)
        {
            iterations = 1;
        }

        // execute the program a number of times		
        for (int i = 0; i < iterations; i++)
        {
            // create a record for use
            jsr = new JavaStringRecord(encoding);
            // set input data if we have any        
            if (commareaLength > 0)
            {
                jsr.setText(commarea);
            }

            // make the call
            try
            {
                eciInt.execute(eSpec, jsr, jsr);
            }
            catch (ResourceException e)
            {
                // output the stacktrace and re-throw it back to the client
                e.printStackTrace();
                dropConnection();
                throw e;
            }
        }
        
        dropConnection();
        
        // return the commarea from the last executed program
      	ResultsBean resultsB = new ResultsBean();
       	resultsB.setResultsString(jsr.getText());
       	resultsB.setResultsBytes(jsr.getData());
       	
       	return resultsB;
    }
    
    /**
     * Get a connection to the EIS using the resource adapter.
     */
    private void getConnection() throws Exception
    {
        // we need to get the Connection Factory and from there a 
        // connection and interaction. So We lookup an ECI Connection Factory
        // (You will need to ensure that you setup your environment so that
        // this JNDI name refers to the required ECI Resource Adapter deployed), 
        // get a connection from the connection factory and then get an 
        // interaction
        ConnectionFactory cf = null;
        try
        {
            javax.naming.Context ic = new InitialContext();
            cf = (ConnectionFactory) ic.lookup("java:comp/env/ECI");
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new Exception(
                "Lookup for java:comp/env/ECI failed. Exception: " + e.toString());
        }
        if (cf == null)
        {
            throw new Exception("Lookup for java:comp/env/ECI resulted in a null reference");
        }
        try
        {
            eciConn = (Connection) cf.getConnection();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new Exception(
                "failed to get connection from ECI Factory. Exception: "
                    + e.toString());
        }
        
        try
        {
            eciInt = (Interaction) eciConn.createInteraction();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new Exception(
                "failed to get interaction from ECI Connection. Exception: "
                    + e.toString());
        }
    }
    
    /**
     *  drops the connection to the EIS
     */
    private void dropConnection()
    {
        // tidy up the interaction and connection and set our references to 
        // null.
        try
        {
            eciInt.close();
            eciConn.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        eciInt = null;
        eciConn = null;
    }
    
    /**
     * Get a connection to the EIS using an unmanaged connection factory.
     */
    private void getNonManagedConnection(
        String username,
        String password,
        String gatewayURL,
        int gatewayPort,
        String cicsServer,
        String mirror,
        int trace)
        throws Exception
    {
        try
        {
            ECIManagedConnectionFactory mcf = new ECIManagedConnectionFactory();
            
            // set details
            mcf.setConnectionURL(gatewayURL);
            mcf.setServerName(cicsServer);
            mcf.setPortNumber(Integer.toString(gatewayPort));
            
            if (username.equals("") == false)
            {
                mcf.setUserName(username);
            }
            if (password.equals("") == false)
            {
                mcf.setPassword(password);
            }
            if (mirror.equals("") == false)
            {
                mcf.setTranName(mirror);
            }
            
            // set CCI trace
            switch (trace)
            {
                case 0 :
                    mcf.setTraceLevel(new Integer(mcf.RAS_TRACE_OFF));
                    break;
                case 1 :
                    mcf.setTraceLevel(new Integer(mcf.RAS_TRACE_ERROR_EXCEPTION));
                    break;
                case 2 :
                    mcf.setTraceLevel(new Integer(mcf.RAS_TRACE_ENTRY_EXIT));
                    break;
                case 3 :
                    mcf.setTraceLevel(new Integer(mcf.RAS_TRACE_INTERNAL));
                    break;
                default :
            }
            
            // set trace output to stderr            
            mcf.setLogWriter(new PrintWriter(System.err));
            
            ConnectionFactory cf = (ConnectionFactory) mcf.createConnectionFactory();
            eciConn = cf.getConnection();
            eciInt = eciConn.createInteraction();
        }
        catch (Throwable t)
        {
            t.printStackTrace();
        }
    }
}