/*
	JavaStringRecord
	
	Modified from CTG sample to add byte[] getBinaryData() method.

*/
package itso.cics.eci.j2ee.testercci;

import java.io.*;

/**
 * This class provides a simple example of using the Streamable interface 
 * This custom record can be used with the ECI Resource adapter
 */
public class JavaStringRecord implements javax.resource.cci.Streamable, javax.resource.cci.Record 
{

    //-------------------------------------------------------------------------
    // Internal properties
    //-------------------------------------------------------------------------
    private String recordName;
    private String desc;
    private String contents = new String("");
    private String enc = "IBM037";             // default commarea text encoding
	private byte[] byteContents = null;

	public JavaStringRecord (String encoding) 
	{
		enc = encoding;
	}

    //-------------------------------------------------------------------------
    //  the following methods are required for the Record interface
    //-------------------------------------------------------------------------
    /**
     * get the name of the Record
     * 
     * @return a String representing the Record Name
     */
    public java.lang.String getRecordName() {
        return recordName;
    }

    /**
     * set the name of the Record
     * 
     * @param newName The Name of the Record
     */
    public void setRecordName(java.lang.String newName) {
        recordName = newName;
    }


    /**
     * set the record short destription
     *
     * @param newDesc The Description for this record
     */
    public void setRecordShortDescription(java.lang.String newDesc) {
        desc = newDesc;
    }

    /**
     * get the short description for this Record
     * 
     * @return a String representing the Description
     */
    public java.lang.String getRecordShortDescription() {
        return desc;
    }

    /**
     * return a hashcode for this object
     *
     * @return hashcode
     */
    public int hashCode() {
        if (contents != null) {
            return contents.hashCode();
        }
        else {
            return super.hashCode();
        }
    }

    /**
     * determines if objects are equal. Objects are equal if they
     * have the same reference or the text contained is identical
     *
     * @return flag indicating true or false
     */
    public boolean equals(java.lang.Object comp) {
        if (!(comp instanceof JavaStringRecord)) {
            return false;
        }

        if (comp == this) {
            return true;
        }
        JavaStringRecord realComp = (JavaStringRecord)comp;
        return realComp.getText().equals(this.getText());
    }

    /**
     * use the superclass clone method for this class
     */
    public java.lang.Object clone() throws CloneNotSupportedException {
        return super.clone();
    }

    //-------------------------------------------------------------------------
    // These methods are required for the streamable interface
    //-------------------------------------------------------------------------
    /**
     * This method is invoked by the ECI Resource Adapter when it is
     * transmitting data to the Record. This means we are receiving a commarea
     * back from the ECI Resource Adapter and must have been passed as an 
     * output record
     * 
     * @param is     The inputStream where we read the information from
     * @exception IOException if an exception occurs on the stream
     */
    public void read(java.io.InputStream is) {
        try {
            int total = is.available();
            byte[] bytes = new byte[total];
            is.read(bytes);

			// set byte contents to the byte array            
            byteContents = bytes;

            //-----------------------------------------------------------------
            // convert the bytes to a string based on the selected encoding
            //-----------------------------------------------------------------
            contents = new String(bytes, enc);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * This method is invoked by the ECI Resource Adapter when it wants to
     * read the record. This means we have been passed as an input record
     * 
     * @param os     The output stream to write the information to
     * @exception IOException if an exception occurs on the stream
     */
    public void write(java.io.OutputStream os) {
        try {

            //-----------------------------------------------------------------
            // output the string as bytes in the selected encoding
            //-----------------------------------------------------------------
            os.write(contents.getBytes(enc));
        }
        catch(Exception e){
            e.printStackTrace();
        }

    }

    /**
     * return the text of this java record
     *
     * @return The text
     */
    public String getText() {
        return contents;
    }    


    /**
     * set the text for this java record
     * 
     * @param newStr The new text
     */
    public void setText(String newStr) {
        contents = newStr;
    }

    /**
     * return the current java encoding used for this record
     *
     * @return the java encoding
     */
    public String getEncoding() {
        return enc;
    }

    /**
     * set the java encoding to be used for this record
     * <p>
     * note no checks are made at this time to see if the encoding is a valid
     * java encoding. If you wish you can modify the code to
     * include this.
     *
     * @param newEnc the new java encoding
     */
    public void setEncoding(String newEnc) {
        enc = newEnc;
    }
    
    // These methods are additional, to expose extra functionality

	/**
	 * return the byte array with the binary data for this java record
	 */
    public byte[] getData()    
    {
    	// note this exposes the underlying private byte[] to the caller.
    	// for safety we should really return a reference to a copy of the array.
    	return byteContents;
    }
    
}
