package itso.cics.eci.testereci;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.*;

import com.ibm.ctg.client.*;

import java.io.PrintWriter;
import java.io.IOException;
import java.util.Date;
import java.util.StringTokenizer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

/**
 * Sends an ECI request to a CICS server using the CTG Java class library.
 * Routes to the appropriate jsp to present the results data.
 * @version 1.05
 */
public class CTGTesterECIServlet extends HttpServlet 
{
	// date loaded into application server
	private Date loadedDate;
	
	// jsp names
	private static final String jspResults = "results.jsp";
	private static final String jspError = "error.jsp";
	private static final String jspException = "exception.jsp";
	
	// names of attributes set in the request
	private static final String attrResultsString = "results";
	private static final String attrDefaultResultsString = "defaultResults";
	private static final String attrHexResultsString = "hexResults";
	private static final String attrErrorMessages = "errorMessages";
	private static final String attrErrorException = "errorException";
	private static final String attrLastLoaded = "lastLoaded";
	private static final String attrCicsProgram = "funcName";
	private static final String attrGatewayURL = "gatewayURL";
	private static final String attrGatewayPort = "gatewayPort";
	private static final String attrServer = "server";
	private static final String attrUsername = "username";
	private static final String attrHttpUsername = "httpusername";
	private static final String attrMirror = "mirror";
	private static final String attrCommareaLength = "commareaLength";
	private static final String attrCommarea = "commareaInput";
	private static final String attrManaged = "managed";
	private static final String attrEncoding = "encoding";
	private static final String attrReturnCode = "returnCode";
	private static final String attrReturnString = "returnString";
	private static final String attrAbendCode = "abendCode";
	private static final String attrTrace = "trace";
	private static final String attrProgress = "progress";

	/**
	 * Initialise the servlet.
	 */
	public void init(ServletConfig sc) throws ServletException
	{
		// call HttpServlet init method
		super.init(sc);
		
		// Set time of loading
		loadedDate = new Date();

	}
	
	/**
	 * Do a GET request. Hand over to common processing routine.
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException 
	{
		try 
		{
			processRequest(request, response);
		}
		catch (Throwable e) 
		{
			e.printStackTrace();
		}		
		
	}

	/**
	 * Do a POST request. Hand over to common processing routine.
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException 
	{
		try 
		{
			processRequest(request, response);
		}
		catch (Throwable e) 
		{
			e.printStackTrace();
		}		
		
	}

	/**
	 * Common processing routine.
	 */		
	public void processRequest(HttpServletRequest request, HttpServletResponse response)
		throws IOException 
	{

		JavaGateway jgaConnection = null;

		// input parameters
		String funcName = null;
		String encoding = null;
		String commareaInput = null;
		String commareaLength = null;
		String username = "";
		String password = "";
		String gatewayURL = "";
		String gatewayPort = "";
		String cicsServer = "";			
		String mirror = "";
		String trace = "";

		String messages = "";
		String progress = "";

		// the jsp to forward to for displaying the results
		String jsp = jspResults;

		// values for strings to be converted into
		int commareaLengthInt = 0;						
		int gatewayDaemonPort = 2006;

		// retrieve parameters
		funcName = request.getParameter("funcName");
		encoding = request.getParameter("encoding");
		commareaInput = request.getParameter("commareaInput");
		commareaLength = request.getParameter("commareaLength");
		username = request.getParameter("username");
		password = request.getParameter("password");
		gatewayURL = request.getParameter("gatewayURL");
		gatewayPort = request.getParameter("gatewayPort");
		cicsServer = request.getParameter("server");
		mirror = request.getParameter("mirror");
		trace = request.getParameter("trace");

		// set username and password to null if empty
		if(username.equals(""))
		{
			username = null;
		}
		if(password.equals(""))
		{
			password = null;
		}

		// set last loaded date
		request.setAttribute(attrLastLoaded, loadedDate.toString());

		// set request details
		request.setAttribute(attrUsername, username);
		request.setAttribute(attrGatewayURL, gatewayURL);
		request.setAttribute(attrServer, cicsServer);
		request.setAttribute(attrTrace, trace);
		request.setAttribute(attrCicsProgram, funcName);
		request.setAttribute(attrEncoding, encoding);
		request.setAttribute(attrCommarea, commareaInput);

			
		// convert commarea length to int
		try
		{
			commareaLengthInt = Integer.parseInt(commareaLength);
			
			// set commarea length attribute
			request.setAttribute(attrCommareaLength, Integer.toString(commareaLengthInt));
		}
		catch(NumberFormatException ex)
		{
			messages += "Commarea length not a number<br>";
		}
			

		// convert port to int
		try
		{
			gatewayDaemonPort = Integer.parseInt(gatewayPort);
			
			// set gateway port attribute
			request.setAttribute(attrGatewayPort, Integer.toString(gatewayDaemonPort));
		}
		catch(NumberFormatException ex)
		{
			messages += "Gateway daemon port not a number<br>";
		}

		// set error messages			
		request.setAttribute(attrErrorMessages, messages);

		// get username and password from basic authentication mechanism if present
 		String encodedString = request.getHeader("authorization");
   		if(encodedString != null) 
   		{
	    	StringTokenizer stz = new StringTokenizer(encodedString);
	   		StringBuffer passwordTokens = new StringBuffer();
    		String schemeType = "";
  	 		String userPassEncoded;
   			byte userPassBytes[];
   			
   			String httpUsername = null;
   			String httpPassword = null;

       		schemeType = stz.nextToken();
     		userPassEncoded = stz.nextToken();
    		userPassBytes = Base64.decode(userPassEncoded);
	  		for (int i = 0; i < userPassBytes.length; i++)
   			{
   		   		passwordTokens.append((char) userPassBytes[i]);
   			}
   			StringTokenizer st = new StringTokenizer(passwordTokens.toString(), ":");
   			if (st.hasMoreTokens()) 
   			{
   		 		httpUsername = st.nextToken();
   			}
  			if (st.hasMoreTokens()) 
  			{
   		 		httpPassword = st.nextToken();
   			}

			// update progress message, with username if found
   			progress += "Username ";
   			if(httpUsername != null)
   			{
   				progress += "(" + httpUsername + ")";
   				
 	  			// set http username in request
	   			request.setAttribute(attrHttpUsername, httpUsername);
   			}
   			progress += " and password found in HTTP basic authn<BR>";
			
			// set flowed username and password to http ones
//			username = httpUsername;
//			password = httpPassword;
		}
		else
		{
   			progress += "No HTTP basic authorization header<BR>";
 		}

		// wrap Gateway code in a try catch block
		try
		{
			if(trace.equals("on"))
			{
				T.setDebugOn(true);
			}
			else
			{
				T.setDebugOn(false);
			}

			// indicate progress
			progress += "Connecting to Gateway<BR>";
						
			// create and open the CICS Gateway
			jgaConnection = new JavaGateway(gatewayURL, gatewayDaemonPort);
	
			ECIRequest eciRequest = null;

			// Need to convert commarea string to byte array in correct codepage.
			byte[] abCommarea = new byte[commareaLengthInt];
			byte[] temp = commareaInput.getBytes(encoding);
			System.arraycopy(temp, 0, abCommarea, 0, temp.length);


			// create the ECI Request
			eciRequest =
			new ECIRequest(
				ECIRequest.ECI_SYNC,
				cicsServer,
				username,
				password,
				funcName,
				mirror,
				abCommarea,
				commareaLengthInt,
				ECIRequest.ECI_NO_EXTEND,
				ECIRequest.ECI_LUW_NEW);

			// indicate progess
			progress += "Flowing ECI request to server<BR>";
			
			// send the request to CICS
			jgaConnection.flow(eciRequest); 

			// check the return code
			if(eciRequest.getCicsRc() == ECIRequest.ECI_NO_ERROR) 
			{
				if (eciRequest.Commarea != null) 
				{
					// set result string attribute
					request.setAttribute(attrResultsString, new String(eciRequest.Commarea, encoding));
				
					// set result string in default encoding attribute
					request.setAttribute(attrDefaultResultsString, new String(eciRequest.Commarea));
				
					// set result string in hex encoding attribute
					request.setAttribute(attrHexResultsString, byteArrayToHex(eciRequest.Commarea));
				}
			} 
			else
			{
				// change jsp to use to the error jsp
				jsp = jspError;
			} 

			// set return codes
			request.setAttribute(attrReturnCode, Integer.toString(eciRequest.getCicsRc()));
			request.setAttribute(attrReturnString, eciRequest.getCicsRcString());
			request.setAttribute(attrAbendCode, eciRequest.Abend_Code);

		}
		catch(Exception ex)
		{
			// exception occured, set exception message in request with timestamp
			String nowStr = "";
			try
			{
				SimpleDateFormat sdf = (SimpleDateFormat)DateFormat.getTimeInstance();
				sdf.applyPattern("MM/dd/yy HH:mm:ss:SSS");
				nowStr = sdf.format(new Date());
			}
			catch(ClassCastException ex2)
			{
				// no SimpleDateFormat instance for this locale
				
				// format with Date toString
				nowStr = (new Date()).toString();
			}
			request.setAttribute(attrErrorException, nowStr + " : " + ex);

			// change jsp to use the exception jsp
			jsp = jspException;
		}
		finally
		{
			// indicate progess
			progress += "Closing Gateway<BR>";
			
			// close gateway
			jgaConnection.close();
		}

		// set progress attribute
		request.setAttribute(attrProgress, progress);
					
		// route to jsp				
		ServletContext servletContext = getServletContext();
		RequestDispatcher dispatcher = servletContext.getRequestDispatcher(jsp);
		try
		{
			dispatcher.forward(request, response);
		}
		catch(ServletException ex)
		{
			// log exception to servlet log
			servletContext.log("Exception routing to jsp: " + ex);
		}


	}

	/** 
	 * convert a byte array to a string representation of the hex value.
	 */
	private String byteArrayToHex(byte[] data)
	{
		StringBuffer hex = new StringBuffer();
		
		for(int i = 0; i < data.length; i++)
		{
			String hexByte = Integer.toHexString(data[i]);
			if(hexByte.length() == 1)
			{
				hexByte = "0" + hexByte;
			} 
			else if(hexByte.length() > 2)
			{
				hexByte = hexByte.substring(hexByte.length() - 2);
			}
			hex.append(hexByte.toUpperCase());
			hex.append(' ');
		}
		
		return hex.toString();

	}

}
