package itso.cics.epi;
import java.io.IOException;

import com.ibm.ctg.epi.AID;
import com.ibm.ctg.epi.EPIException;
import com.ibm.ctg.epi.EPIGateway;
import com.ibm.ctg.epi.EPISecurityException;
import com.ibm.ctg.epi.Field;
import com.ibm.ctg.epi.Screen;
import com.ibm.ctg.epi.Terminal;

public class SignonCapable {
	/**
	 * EpiSimple constructor comment.
	 */
	public SignonCapable() {
		super();
	}

	private final static String DEFAULT_URL = "local";
	private final static int DEFAULT_PORT = 2006;
	private final static String DEFAULT_REGION = "SCSCPJA4";
	static String userid = null;
	static String password = null;
	static String cicsRegion = null;
	static EPIGateway epiGate;

	private boolean parseCommandLine(String[] args) throws IOException {

		switch (args.length) {

			case 5 :

				{
					System.out.println("Please only 4 input parameters as follows:");
					System.out.println(
						"Usage: java"
							+ " SignonCapable"
							+ " [GatewayUrl]"
							+ " [region]"
							+ " [userid]"
							+ " [password]");
					return false;
				}
			case 4 :
				password = args[3].trim();
				userid = args[2].trim();
				cicsRegion = args[1].trim();
				epiGate.setURL(args[0].trim());
				break;
			case 3 :
				cicsRegion = DEFAULT_REGION;
				password = args[3].trim();
				userid = args[2].trim();
				cicsRegion = args[1].trim();
				epiGate.setURL(args[0].trim());
				break;
			case 2 :
				userid = args[2].trim();
				cicsRegion = args[1].trim();
				epiGate.setURL(args[0].trim());
				break;
			case 1 :
				cicsRegion = DEFAULT_REGION;
				epiGate.setURL(DEFAULT_URL);
				break;
			default :

				System.out.println("Please supply input parameters as follows:");
				System.out.println(
					"Usage: java"
						+ " SignonCapable"
						+ " [GatewayUrl]"
						+ " [region]"
						+ " [userid]"
						+ " [password]");
				return false;
		}
		return true;

	}

	public static void main(java.lang.String[] args) {

		SignonCapable sample = new SignonCapable();

		try {
			epiGate = new EPIGateway();
			Field fld = null;

			if (sample.parseCommandLine(args)) {
				System.out.println("Gateway URL: " + epiGate.getURL());
				System.out.println("Region: " + cicsRegion);
				System.out.println("Userid: " + userid);
				System.out.println("Password: " + password + "\n");
				
				epiGate.open();
			}
			
			String termUserid = null; 
			String termPassword = null;

			Terminal term =
				new Terminal(
					epiGate,
					cicsRegion,
					null,
					null,
					Terminal.EPI_SIGNON_CAPABLE,
					termUserid,
					termPassword,
					0,
					null);
			System.out.println("Adding signon capable terminal with userid:" + termUserid);
			term.connect();

			System.out.println("Netname: " + term.getNetName());

			Screen scr = term.getScreen();
			scr.field(1).setText("CESN");
			scr.setAID(AID.enter);
			System.out.println("Starting CESN");
			term.send();
			/*for (int i = 1; i <= scr.fieldCount(); i++) {
				fld = scr.field(i); // get field by index
				if (fld.textLength() > 0)
					System.out.println("Field " + i + ":" + fld.getText());
			}*/


			if (scr.field(1).getText().startsWith("DFHCE3547")) 
			{
				System.out.println("ERROR: CESN failure");
				System.out.println(scr.field(1).getText());
				throw new EPIException("CESN Error: DFHCE3547");
			}

			scr.field(10).setText(userid);
			scr.field(16).setText(password);
			System.out.println("Entering userid and password");
			term.send();
			
			if (scr.field(1).getText().startsWith("DFHCE3549")) {
			  System.out.println(" DFHCE3549: Now signed on to CICS");
			}
			else {
				throw new EPIException("CESN Error: Signon failed");
			}

	
			scr.setAID(AID.PF3);
			term.send();

			scr.field(1).setText("EPIP");
			scr.setAID(AID.enter);
			System.out.println("Starting EPIP");
			term.send();

			EPIMAPMap epimap = new EPIMAPMap(scr);
			System.out.println("EPIP results");
			System.out.println(" Time : " + epimap.field(epimap.TIME).getText());
			System.out.println(" Date : " + epimap.field(epimap.DATE).getText());
			System.out.println(" Applid : " + epimap.field(epimap.APPLID).getText());
			System.out.println(" Userid : " + epimap.field(epimap.USERID).getText());


			System.out.println("Closing terminal and gateway");
			term.disconnect();
			epiGate.close();

		} catch (EPISecurityException epiSecEx) {
			if (epiSecEx.getErrorCode() == EPISecurityException.ESI_ERR_PASSWORD_INVALID) {
				System.out.println("ERROR: Invalid password was supplied");
			} else 
				System.out.println("ERROR: Unknown security error encountered");

		} catch (EPIException epiEx) {
			System.out.println("ERROR: Unknown EPI error encountered");
			System.out.println("Error message was :" + epiEx.getMessage());
			epiEx.printStackTrace();

		} catch (java.io.IOException ioEx) {
			ioEx.printStackTrace();
		}
	}
}