package itso.cics.epi;

import com.ibm.ctg.epi.*;
import itso.cics.epi.EPIMAPMap;
import java.io.IOException;

public class SignonIncapable {
	/**
	 * EpiSimple constructor comment.
	 */
	public SignonIncapable() { 
		super();
	}

	private final static String DEFAULT_URL = "local";
	private final static int DEFAULT_PORT = 2006;
	private final static String DEFAULT_REGION = "SCSCPJA4";
	static String userid = null;
	static String password = null;
	static String cicsRegion = null;
	static EPIGateway epiGate;

	private boolean parseCommandLine(String[] args) throws IOException {

		switch (args.length) {

			case 5 :
				{
					System.out.println("Please only 4 input parameters as follows:");
					System.out.println(
						"Usage: java"
							+ " SignonIncapable"
							+ " [GatewayUrl]"
							+ " [region]"
							+ " [userid]"
							+ " [password]");
					return false;
				}
			case 4 :
				password = args[3].trim();
				userid = args[2].trim();
				cicsRegion = args[1].trim();
				epiGate.setURL(args[0].trim());
				break;
			case 3 :
				cicsRegion = DEFAULT_REGION;
				password = args[3].trim();
				userid = args[2].trim();
				cicsRegion = args[1].trim();
				epiGate.setURL(args[0].trim());
				break;
			case 2 :
				userid = args[2].trim();
				cicsRegion = args[1].trim();
				epiGate.setURL(args[0].trim());
				break;
			case 1 :
				cicsRegion = DEFAULT_REGION;
				epiGate.setURL(DEFAULT_URL);
				break;
			default :

				System.out.println("Please supply input parameters as follows:");
				System.out.println(
					"Usage: java"
						+ " SignonIncapable"
						+ " [GatewayUrl]"
						+ " [region]"
						+ " [userid]"
						+ " [password]");
				return false;
		}
		return true;

	}

	public static void main(java.lang.String[] args) {

		SignonIncapable sample = new SignonIncapable();

		try {
			epiGate = new EPIGateway();

			if (sample.parseCommandLine(args)) {
				System.out.println("Gateway URL: " + epiGate.getURL());
				System.out.println("Region: " + cicsRegion);
				System.out.println("Userid: " + userid);
				System.out.println("Password: " + password + "\n");

				epiGate.open();
			}

			Terminal term =
				new Terminal(
					epiGate,
					cicsRegion,
					null,
					null,
					Terminal.EPI_SIGNON_INCAPABLE,
					userid,  //userid
					password,  //password
					0,
					null);
			System.out.println("Adding signon incapable terminal");
			term.connect();
			
			System.out.println ("Netname: " + term.getNetName());
			
			Screen scr = term.getScreen();

			term.setUserid(userid);
			term.setPassword(password);

			scr.field(1).setText("EPIP");
			scr.setAID(AID.enter);
			System.out.println("Starting EPIP");
			term.send();

			EPIMAPMap epimap = new EPIMAPMap(scr);
			System.out.println(" Time : " + epimap.field(epimap.TIME).getText());
			System.out.println(" Date : " + epimap.field(epimap.DATE).getText());
			System.out.println(" Applid : " + epimap.field(epimap.APPLID).getText());
			System.out.println(" Userid : " + epimap.field(epimap.USERID).getText());

			System.out.println("Closing terminal and gateway");
			term.disconnect();
			epiGate.close();

		} catch (EPIException epiEx) {
			epiEx.printStackTrace();
		} catch (java.io.IOException ioEx) {
			ioEx.printStackTrace();
		}
	}
}